<?php
/**
 * @file
 * Drupal Client for TenonIO web service.
 * See http://www.tenon.io/documentation/.
 *
 * Main wrapper around webservice.
 */

namespace lib\TenonIO\Client;


/**
 * TenonIO Client
 *
 * This is a Drupal client for the TenonIO web service.
 * See http://www.tenon.io/documentation/
 * with some additional methods thrown in for good measure.
 */
class TenonIODrupalClient {

  protected $tasks;
  protected $service_url;
  protected $key;



  /**
   * Constructor.
   */
  public function __construct($service_url) {
    $this->service_url = $service_url;
    $this->key = variable_get('tenon_io_api_key', 'bc6d7331703a033ca3cbd1fc7d6b7d43');

    // Loop through DB registry to reload tasks already entered.
    $tasks = db_select('tenon_io_task_registry', 'p')->fields('p', array('task_id', 'created', 'result', 'has_run', 'url', 'nid'))->execute();

    foreach ($tasks as $task) {
      $this->addTask($task);
    }
  }

  /**
   * Get all tasks.
   */
  public function getTasks() {

    return $this->tasks;
  }

  /**
   * Get full results for all tasks.
   */
  public function getTasksResults() {

    if (empty($this->tasks)) {
      drupal_set_message(t('No tasks were found, please run a <a href="/admin/config/content/tenon_io/settings">Test</a> first'), 'warning');
      return '';
    }

    foreach ($this->tasks as $task) {

      $task->date = date('Y-m-d H:i:s', $this->tasks[$task->task_id]->created);
      $return[] = $task;
    }


    return $return;
  }

  /**
   * Get a particular task.
   */
  public function getTask($task_id) {

    return $this->tasks[$task_id];
  }

  /**
   * Delete all tasks.
   */
  public function deleteTasks() {
    foreach ($this->tasks as $task) {

      $this->deleteTask($task);
    }
  }

  /**
   * Delete a task.
   */
  public function deleteTask($task_id) {

    if (db_delete('tenon_io_task_registry', array('task_id' => $task_id))->execute()) {
      unset($this->tasks[$task_id]);
    }

  }

  /**
   * Run a task.
   */
  public function runTask(&$task, $standard = 'AAA') {

    if (is_numeric($task)) {
      $task = $this->getTask($task);
    }

    $options = array('key' => $this->key);

    if (!variable_get('tenon_io_firewall_mode', 0)) {
      $options['url'] = $task->url;
    }
    else{ // If we're in firewall mode, submit the page as a whole.
      /*TODO work out what to do with js/css */
      $page = drupal_http_request($task->url);
      if (!$page->error) {
        $options['src'] = $page->data;
      }
      else{
         throw new Exception('Tenon.io could not run task, received error: ' . $page->error);
      }
    }

    $request = new \lib\TenonIO\Request\TenonIORequest($options);

    $return = drupal_http_request(
        variable_get('tenon_io_url', 'http://tenon.io/api'),
          array(
            'method' => 'POST',
            'data' => drupal_http_build_query(
              array(
                //cast as an array, to convert to string..
                (array) $request
              )
             ),
            'timeout' => 15,
            'headers' => array('Content-Type' => 'application/x-www-form-urlencoded'),
            )
        );
    if ($return->code == 200) {

      $task->result = $return->data;
      return TRUE;

    }

    return FALSE;
  }

  /**
   * Get a result for a task.
   */
  public function getTaskResult($task_id, $result_id) {
    return $this->tasks['task_id']->result;
  }

  /**
   * Add a task to the internal task registry.
   */
  protected function addTask($task) {
    if (!isset($task->task_id)) {
      throw new \Exception('No id found for task: ' . var_export($task, TRUE));
    }

    $this->tasks[$task->task_id] = $task;
  }
}
