<?php
/**
 * @file
 * Administrative functions.
 *
 * Contains administrative functions.
 */

/**
 * Form constructor for the admin settings form.
 *
 * @see tenon_io_menu()
 */
function tenon_io_admin_settings_form(array $form, array $form_state) {

  $form['tenon_io_settings']['#tree'] = TRUE;

  $form['tenon_io_url'] = array(
    '#type' => 'textfield',
    '#required' => TRUE,
    '#title' => t('Tenon.io webservices url'),
    '#description' => t('Tenon.io webservices url'),
    '#default_value' => variable_get('tenon_io_url', 'http://www.tenon.io/api/'),
  );

  $form['tenon_io_api_key'] = array(
    '#type' => 'textfield',
    '#required' => TRUE,
    '#title' => t('Tenon.io webservices API key'),
    '#description' => t('Tenon.io API Key'),
    '#default_value' => variable_get('tenon_io_api_key', '[ADD YOUR KEY]'),
  );

  $form['tenon_io_test_standard'] = array(
    '#type' => 'select',
    '#required' => TRUE,
    '#title' => t('TENON_IO test standard'),
    '#description' => t('TENON_IO test standard to use for testing'),
    '#options' => array(
      'A' => t('WCAG2A'),
      'AA' => t('WCAG2AA'),
      'WCAG2AAA' => t('WCAG2AAA')),
    '#default_value' => variable_get('tenon_io_test_standard', 'WCAG2AA'),
  );

  $form['tenon_io_run_cron_limit'] = array(
    '#type' => 'select',
    '#required' => TRUE,
    '#title' => t('Tenon.io run cron limit'),
    '#description' => t(
        'Limit on how many untested node to submit for test on each cron run.'),
    '#options' => range(0, 50, TENON_IO_INTEGRATION_LIMIT_INCREMENT),
    '#default_value' => variable_get('tenon_io_run_cron_limit', 10),
  );

  $form['tenon_io_show_on_page'] = array(
    '#type' => 'checkbox',
    '#title' => t('Tenon.io show results on page'),
    '#description' => t(
        'For all users with the tenon.io %url permission, show results on each page as well as a button to (Re)test.',
          array(
            '%url' => l(t('show result on page'),
            'admin/people/permissions/tenon_io'))
    ),
    '#default_value' => variable_get('tenon_io_show_on_page', 0),
  );

  $form['tenon_io_firewall_mode'] = array(
    '#type' => 'checkbox',
    '#title' => t('Tenon.io Firewall mode'),
    '#description' => t(
        'Send the content of the page, rather than the url to Tenon.io. Use this if you are behind a firewall or NAT so tenon.io cannot actually connect to your site to do the audit.'),
    '#default_value' => variable_get('tenon_io_firewall_mode', 0),
  );

  $form['tenon_io_test'] = array(
    '#title' => t('Start AX testing'),
    '#type' => 'submit',
    '#description' => '<p>' . t(
        'To test every page on this site for accessibility, please press the Start button below') . '</p>',
    '#prefix' => '<div>',
    '#value' => t('Start Tenon.io test'),
    '#submit' => array('tenon_io_admin_settings_form_test_submit'),

  );

  $form['tenon_io_delete'] = array(
    '#title' => t('Delete all tests'),
    '#type' => 'submit',
    '#value' => t('Delete all Tenon.io tests'),
    '#description' => '<p>' . t('To test every page on this site for accessibility, please press the Start button below') . '</p>',
    '#submit' => array('tenon_io_admin_settings_form_delete_submit'),
  );

  $form = system_settings_form($form);

  return $form;
}

/**
 * Form validation handler for tenon_io_admin_settings_form().
 *
 * @see tenon_io_admin_settings_form()
 */
function tenon_io_admin_settings_form_validate(array $form, array $form_state) {

  $parsed_url = parse_url($form_state['values']['tenon_io_url']);
  if (!($parsed_url['scheme'] == 'http' || $parsed_url['scheme'] == 'https' && isset($parsed_url['host']))) {
    form_set_error('tenon_io_url', t('TENON_IO Integration url set is not a valid url'));
  }
}
/**
 * Form submit function for test form.
 *
 * @see tenon_io_menu()
 */
function tenon_io_admin_settings_form_test_submit(array $form, array &$form_state) {
  $destination = array();
  // Check url is valid.
  if ($test_request = drupal_http_request(variable_get('tenon_io_url', ''), array('method' => 'POST'))) {

    if ($test_request->code != 200) {
      form_set_error('tenon_io_url', t(
          'Tenon IO Webservice is not responding, perhaps url is not set or invalid?')
      );
      drupal_goto('admin/config/content/tenon_io');
    }
  }
  if (isset($_GET['destination'])) {
    $destination = drupal_get_destination();
    unset($_GET['destination']);
  }
  $form_state['redirect'] = array('admin/config/content/tenon_io/test', array('query' => $destination));
}
/**
 * Form submit function for delete form..
 *
 * @see tenon_io_menu()
 */
function tenon_io_admin_settings_form_delete_submit(array $form, array &$form_state) {
  $destination = array();
  if (isset($_GET['destination'])) {
    $destination = drupal_get_destination();
    unset($_GET['destination']);
  }
  $form_state['redirect'] = array('admin/config/content/tenon_io/delete', array('query' => $destination));

}

/**
 * Form builder for the start test form.
 */
function tenon_io_admin_settings_form_test_confirm(array $form, array &$form_state) {

  $form['delete_all'] = array(
    '#type' => 'checkbox',
    '#title' => t('Delete all existing results first'),
    '#description' => t('Before testing, delete all existing results first.'),
    '#default_value' => 0,
  );

  return confirm_form($form,
    t('Are you sure you want to start testing your site with Tenon.io?'),
    'admin/config/content/tenon_io/test/start',
    t('All publicly visible content will be validated using Tenon.io.'),
    t('Test all content')
  );
}

/**
 * Form builder for the delete test form.
 */
function tenon_io_admin_settings_form_delete_confirm(array $form, array &$form_state) {
  return confirm_form($form,
    t('Are you sure you want delete to all tasks registered with Tenon.io?'),
    'admin/config/content/tenon_io/delete/start',
    t('All publicly visible content tasks will be deleted from Tenon.io.'),
    t('Delete all content tasks from Tenon.io')
  );
}
/**
 * Form submit function for test confirm form..
 *
 * @see tenon_io_menu()
 */
function tenon_io_admin_settings_form_test_confirm_submit(array $form, array &$form_state) {

  tenon_io_batch_test($form_state['values']['delete_all']);
}
/**
 * Form submit function for delete confirm form..
 *
 * @see tenon_io_menu()
 */
function tenon_io_admin_settings_form_delete_confirm_submit(array $form, array &$form_state) {
  tenon_io_batch_delete();
}

/**
 * Submit a batch job to test content.
 */
function tenon_io_batch_test($delete_all_first = FALSE) {

  if ($delete_all_first) {
    db_delete('tenon_io_task_registry')->execute();
    drupal_set_message(t('All tenon.io test deleted'), 'notice');
  }

  $result = db_select('node', 'n')->fields('n', array('nid', 'title'))->condition('status', 1)->countQuery()->execute();
  $total_count = $result->fetchAssoc();

  $batch = array(
    'operations' => array(
      array(
        'tenon_io_batch_test_tasks',
        array($total_count['expression']),
      ),
    ),
    'finished' => 'tenon_io_batch_finished',
    'title' => t('Testing'),
    'init_message' => t('Preparing to submit content to Tenon.io for testing...'),
    'progress_message' => t('Submitting content to Tenon.io...'),
    'error_message' => t('Tenon.io test submission has encountered an error.'),
    'file' => drupal_get_path('module', 'tenon_io') . '/tenon_io.admin.inc',
  );
  batch_set($batch);
}


/**
 * Submit a batch job to delete content.
 */
function tenon_io_batch_delete() {
  $batch = array(
    'operations' => array(
      array(
        'tenon_io_batch_delete_tasks',
        array(),
      ),
    ),
    'finished' => 'tenon_io_batch_finished',
    'title' => t('Deleting'),
    'init_message' => t('Preparing to submit content to TENON_IO for removal...'),
    'progress_message' => t('Submitting content to Tenon.io...'),
    'error_message' => t('Tenon.io delete submission has encountered an error.'),
    'file' => drupal_get_path('module', 'tenon_io') . '/tenon_io.admin.inc',
  );
  batch_set($batch);
}

/**
 * Batch Operation Callback.
 *
 * @throws Exception.
 *   When TENON_IO gives an error,
 *   throw an exception that TENON_IO is not available.
 */
function tenon_io_batch_test_tasks($total_limit, &$context) {

  if (empty($context['sandbox'])) {

    try {
      // Get content.
      $context['sandbox']['content'] = tenon_io_get_content();

    }
    catch (Exception $e) {
      watchdog('TENON_IO', $e->getMessage(), NULL, WATCHDOG_ERROR);

    }

    $context['sandbox']['submitted'] = 0;
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['batch_progress'] = 0;
    $context['sandbox']['total'] = $total_limit;
    $context['sandbox']['batch_total'] = count($context['sandbox']['content']);
  }

  try{
    if (tenon_io_submit_task($context['sandbox']['content'][$context['sandbox']['batch_progress']])) {
      $context['sandbox']['submitted']++;
    }
  }
  catch (\Exception $e) {
    watchdog('TENON_IO', $e->getMessage(), NULL, WATCHDOG_ERROR);
  }
  $context['sandbox']['progress']++;
  $context['sandbox']['batch_progress']++;

  $arguments = array(
    '@current' => $context['sandbox']['progress'],
    '@total' => $total_limit,
    '@submitted' => $context['sandbox']['submitted'],
  );

  $context['message'] = t('Inspected @current of @total entities. Submitted @submitted documents to TENON_IO', $arguments);

  // Inform the batch engine that we are not finished, and provide an
  // estimation of the completion level we reached.
  $context['finished'] = empty($context['sandbox']['batch_total']) ? 1 : $context['sandbox']['progress'] / $context['sandbox']['total'];

  // Reset content if we've completed a batch.
  if ($context['sandbox']['batch_total'] == $context['sandbox']['batch_progress'] && $context['sandbox']['batch_total'] < $context['sandbox']['total']) {
    $context['sandbox']['content'] = tenon_io_get_content($context['sandbox']['progress']);
    $context['sandbox']['batch_progress'] = 0;
  }
  // Put the total into the results section when we're finished so we can
  // show it to the admin.
  if ($context['finished']) {
    $context['results']['count'] = $context['sandbox']['progress'];
    $context['results']['submitted'] = $context['sandbox']['submitted'];
    $context['results']['op'] = 'testing';
  }

}

/**
 * Batch Operation Callback.
 *
 * @return false
 *   return false when an exception was caught
 *
 * @throws Exception.
 *   When TENON_IO gives an error,
 *   throw an exception that TENON_IO is not available.
 */
function tenon_io_batch_delete_tasks(&$context) {

  if (empty($context['sandbox'])) {

    try {
      // Get content,
      tenon_io_require_deps();
      $tenon_io = new \lib\TenonIO\Client\TenonIODrupalClient(variable_get('tenon_io_url', 'http://www.tenon.io/api'));
      $context['sandbox']['content'] = array_keys($tenon_io->getTasks());
    }
    catch (Exception $e) {
      watchdog('TENON_IO', $e->getMessage(), NULL, WATCHDOG_ERROR);
      return FALSE;
    }

    $context['sandbox']['submitted'] = 0;
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['total'] = count($context['sandbox']['content']);
  }

  try{
    tenon_io_delete_task($context['sandbox']['content'][$context['sandbox']['progress']]);
  }
  catch (\Exception $e) {
    watchdog('TENON_IO', $e->getMessage(), NULL, WATCHDOG_ERROR);
    return FALSE;
  }

  $context['sandbox']['progress']++;
  $context['sandbox']['submitted']++;

  $arguments = array(
    '@current' => $context['sandbox']['progress'],
    '@total' => $context['sandbox']['total'],
    '@submitted' => $context['sandbox']['submitted'],
  );

  $context['message'] = t('Removed @current of @total entities', $arguments);

  // Inform the batch engine that we are not finished, and provide an
  // estimation of the completion level we reached.
  $context['finished'] = empty($context['sandbox']['total']) ? 1 : $context['sandbox']['progress'] / $context['sandbox']['total'];

  // Put the total into the results section when we're finished so we can
  // show it to the admin.
  if ($context['finished']) {
    $context['results']['count'] = $context['sandbox']['progress'];
    $context['results']['submitted'] = $context['sandbox']['submitted'];
    $context['results']['op'] = 'deleting';
  }

}

/**
 * Batch 'finished' callback.
 */
function tenon_io_batch_finished($success, array $results, array $operations) {

  $message = '';

  if (isset($results['count'])) {
    $message .= format_plural($results['count'], '1 item processed successfully. ', '@count items successfully processed. ');
  }
  if (isset($results['submitted'])) {
    $message .= format_plural($results['submitted'], '1 document successfully sent to Tenon.io for ' . $results['op'] . '.', '@count documents successfully sent to Tenon.io for ' . $results['op'] . '. For the results, see <a href="!url">the status page</a>', array('!url' => '/admin/reports/tenon_io'));
  }
  if ($success) {
    $type = 'status';
  }
  else {
    // An error occurred. $operations contains the unprocessed operations.
    $error_operation = reset($operations);
    $message .= ' ' . t('An error occurred while processing @num with arguments: @args',
            array(
              '@num' => $error_operation[0],
              '@args' => print_r($error_operation[0], TRUE)));
    $type = 'error';
  }
  drupal_set_message(($message), $type);
  drupal_goto(variable_get('tenon_io_admin_path', 'admin/config/content/tenon_io'));
}
